/* 
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software 
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * -------------------------------------------------------------------------------------------
 * Comedi acquisition server: Comedi module. Prepares and executes comedi functions
 * 
 * Revision 1.4  2009/12/05 13:11:42  elaine
 * Minor enhancements
 *
 * Revision 1.3  2009/10/17 18:17:38  elaine
 * Changed the way the parent process waits for the child to finish.
 *
 * Revision 1.2  2009/10/17 01:45:09  elaine
 * Added error checking for the modprobe and comedi_config calls.
 *
 * Revision 1.1  2009/08/03 01:59:39  elaine
 * Initial revision
 *
 *-------------------------------------------------------------------------------------------
 *
 */

/* required header files */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stddef.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <comedilib.h>     // provides comedi functions
#include "includes/comedisrv.h"     // provides comedi modified functions and constants


extern const char *comediErr;
void execComedi(int newSockfd,char **par,char *sendStatus, char *onStatus){

/* function constants */
#define OPEN                1   //comedi_open()
#define CONFIG              2   //comedi_config()
#define CLOSE               3   //comedi_close() 
#define GETNSUBDEV          4   //comedi_get_n_subdevices()
#define DIOREAD             5   //comedi_dio_read()
#define DIOWRITE            6   //comedi_dio_write()
#define DIOCONFIG           7   //comedi_dio_config()
//#define DIOBITFIELD         8   //comedi_dio_bitfield()
#define DATAREAD            9   //comedi_data_read()
#define DATAWRITE          10   //comedi_data_write()
#define GETMAXDATA         11   //comedi_get_maxdata()
#define TOPHYS             12   //comedi_to_phys()
#define FROMPHYS           13   //comedi_from_phys()
//#define DOINSN             14   //comedi_do_insn()  
//#define COMMAND            15   //comedi_command()
//#define COMMANDTEST        16   //comedi_command_test()
//#define COMMSRCMASK        17   //comedi_get_cmd_src_mask()
#define GETSUBDEVTYPE      18   //comedi_get_subdevice_type()
#define MODPROBE           19   //modprobe
#define GETRANGE           20 //comedi_get_range()
#define GETVER             21 //comedi_get_version_code()
#define GETDRIVER          22 //comedi_get_driver_name()

/* path constants */
#define MOD_PATH "/sbin/modprobe"
#define CONF_PATH "/usr/local/sbin/comedi_config"

	

	/* variables */
	static comedi_t *comedi_device;
	lsampl_t data, maxdata, sampleUnit;
	static comedi_range *rangeType;
	double physUnit;
	unsigned int subdevice,channel, bit;
	int range,errno, aref;
	char *onFailure = "1\n\n";
	char *onSuccess = "0\n\n";
	pid_t pid;           /* child process ID */
	pid_t wpid;          /* wait() process ID */
	int wstatus;         /* wait() status */
	int function,direction, getRangeWasExecuted = 0;
	const char *dev = "/dev/comedi";
	char arg[20];
	
		
	
	// function assignment         
	if(!strcmp(par[1],"comedi_open")){
		function = OPEN;
	}
	else if(!strcmp(par[1],"comedi_close")){
		function = CLOSE;
	}
	else if(!strcmp(par[1],"comedi_config")){
		function = CONFIG;
	}
	else if(!strcmp(par[1],"comedi_get_n_subdevices")){
		function = GETNSUBDEV;
	}
	else if(!strcmp(par[1],"comedi_dio_read")){
		function = DIOREAD;
	}
	else if(!strcmp(par[1],"comedi_dio_write")){
		function = DIOWRITE;
	}
	else if(!strcmp(par[1],"comedi_dio_config")){
		function = DIOCONFIG;
	}
	else if(!strcmp(par[1],"comedi_data_read")){
		function = DATAREAD;
	}
	else if(!strcmp(par[1],"comedi_get_range")){
		function = GETRANGE;
	}
	else if(!strcmp(par[1],"comedi_from_phys")){
		function = FROMPHYS;
	}
	else if(!strcmp(par[1],"comedi_data_write")){
		function = DATAWRITE;
	}
	else if(!strcmp(par[1],"comedi_to_phys")){
		function = TOPHYS;
	}
	else if(!strcmp(par[1],"comedi_get_maxdata")){
		function = GETMAXDATA;
	}
	else if(!strcmp(par[1],"comedi_modprobe")){
		function = MODPROBE;
	}
	else if(!strcmp(par[1],"comedi_get_version_code")){
		function = GETVER;
	}
	else if(!strcmp(par[1],"comedi_get_driver_name")){
		function = GETDRIVER;
	}
	else if(!strcmp(par[1],"comedi_get_subdevice_type")){
		function = GETSUBDEVTYPE;
	}
	else{
		printf("server: function %s is not supported by this server or does not exist.\n",par[1]);
		
	}
	switch(function){


		case OPEN:; // there can only be 1 argument 
			  strcpy(arg,dev);
			  strcat(arg,par[2]); // concatenates device and argument to form "/dev/comediN"
			  arg[13] = '\0';
		
			  comedi_device = comedi_open(arg);

			  if(comedi_device == NULL){
				  clearStr(sendStatus, onStatus);
				  comediErr= comedi_strerror(comedi_errno());
				  strcpy(sendStatus,comediErr);
				  strcpy(onStatus ,onFailure);
				  break;
			  }
			  else{
				  clearStr(sendStatus, onStatus);
				  strcpy(sendStatus,"Comedi input opened successfully\n"); 
				  strcpy(onStatus,onSuccess);
			  }

			  break;

		case DATAREAD:;

			      int datareadstat = 0;

			      // if the device could be opened
			      if(comedi_device != NULL){
				      subdevice = atoi((par[3]));
				      channel = atoi((par[4]));
				      range = atoi((par[5]));
				      aref = atoi((par[6]));
				      			      
				      datareadstat = comedi_data_read(comedi_device,subdevice,channel,range,aref,&data);
				     
				      // if comedi_data_read fails  
				      if (datareadstat < 0){
					      clearStr(sendStatus,onStatus);
					      comediErr = comedi_strerror(comedi_errno());
					      strcpy(sendStatus,comediErr);
					      strcpy(onStatus ,onFailure);	
					      break;
				      }
				      // if comedi_data_read OK, converts and sends message
				      else{

					      clearStr(sendStatus,onStatus);
					      snprintf(sendStatus,BUFFERSIZE,"%d",data); //inserts the content of "data" to the sendStatus string
					      strcpy(onStatus,onSuccess);
					      break;
				      }
			      }
			      // if the device could NOT be opened
			      else{
				      clearStr(sendStatus,onStatus);
				      nullDev(sendStatus, onStatus); // calls the null device error message
				      break;

			      }

			      break;

			      
		case DATAWRITE:;

			       int dataWriteStat = 0;

			       // if the device could be opened
			       if((comedi_device != NULL) && (rangeType != NULL)){

				      subdevice = atoi((par[3]));
				      channel = atoi((par[4]));
				      range = atoi((par[5]));
				      aref = atoi((par[6]));
				      data = atoi(par[7]);

				      dataWriteStat = comedi_data_write(comedi_device,subdevice,channel,range,aref,data);
				      			     
				      // if comedi_data_write fails  
				      if (dataWriteStat < 0){
					      clearStr(sendStatus,onStatus);
					      comediErr = comedi_strerror(comedi_errno());
					      strcpy(sendStatus,comediErr);
					      strcpy(onStatus ,onFailure);
					      break;
				      }
				      // if comedi_data_write OK sends message
				      else{
					      clearStr(sendStatus, onStatus);
					      snprintf(sendStatus,BUFFERSIZE,"%d",data);
					      strcpy(onStatus,onSuccess);
					      break;
				      }
			       }
			       // if the device could NOT be opened
			       else{
				       clearStr(sendStatus,onStatus);
				       nullDev(sendStatus,onStatus); // calls the null device error message
				      break;

			       }

			      break;


		case DIOCONFIG:;


			       int dioConfStat = 0;

			       // if the device could be opened
			      if(comedi_device != NULL){
				      subdevice = atoi((par[3]));
				      channel = atoi((par[4]));
				      direction = atoi((par[5]));

				      if (direction == 0){
					      dioConfStat = comedi_dio_config(comedi_device,subdevice,channel,COMEDI_INPUT);
				      }
				      if (direction == 1){
					      dioConfStat = comedi_dio_config(comedi_device,subdevice, channel, COMEDI_OUTPUT);
				      }

				      // if comedi_dio_config fails  
				      if (dioConfStat < 0){

					      clearStr(sendStatus,onStatus);
					      comediErr = comedi_strerror(comedi_errno());
					      strcpy(sendStatus,comediErr);
					      strcpy(onStatus ,onFailure);	
					      break;
					      }

				      // if comedi_dio_config OK sends message
				      else{
				             clearStr(sendStatus, onStatus);
					     strcpy(sendStatus,"Device configured properly\n");
					     strcpy(onStatus,onSuccess);
					     break;
					     }
				      }
			      // if the device could NOT be opened
			      else{
				      clearStr(sendStatus, onStatus);
				      nullDev(sendStatus, onStatus); // calls the null device error message
				      break;

				      }

			      break;


		case DIOREAD:;


			     int dioReadStat = 0;

			      // if the device could be opened
			      if(comedi_device != NULL){
				      subdevice = atoi((par[3]));
				      channel = atoi((par[4]));
				      

				      dioReadStat = comedi_dio_read(comedi_device,subdevice,channel,&bit);
				     
				      // if comedi_dio_read fails  
				      if (dioReadStat < 0){
					      clearStr(sendStatus, onStatus);
					      comediErr = comedi_strerror(comedi_errno());
					      strcpy(sendStatus,comediErr);
					      strcpy(onStatus ,onFailure);	
					      break;
					      }
				      // if comedi_dio_read OK, converts and sends message
				      else{

					      clearStr(sendStatus, onStatus);
					      snprintf(sendStatus,BUFFERSIZE,"%d",bit); //inserts the content of "bit" to the dioRMsg string
					      strcpy(onStatus,onSuccess);
					      break;
					      }
				      }
			      // if the device could NOT be opened
			      else{
				      clearStr(sendStatus, onStatus);
				      nullDev(sendStatus,onStatus); // calls the null device error message
				      break;
				      }

			      break;


		case DIOWRITE:;

			      int dioWStat = 0;
			      
			      // if the device could be opened
			      if(comedi_device != NULL){
				      subdevice = atoi((par[3]));
				      channel = atoi((par[4]));
				      bit = atoi((par[5]));
				      
				      
				      dioWStat = comedi_dio_write(comedi_device,subdevice,channel,bit);
				      
				      // if comedi_dio_write fails  
				      if (dioWStat < 0){
					      clearStr(sendStatus, onStatus);
					      comediErr = comedi_strerror(comedi_errno());
					      strcpy(sendStatus,comediErr);
					      strcpy(onStatus ,onFailure);	
					      break;
					      }
				      // if comedi_data_read OK, converts and sends message
				      else{
					      clearStr(sendStatus, onStatus);
					      snprintf(sendStatus,BUFFERSIZE,"%d",bit);
					      strcpy(onStatus,onSuccess);
					      break;
					      }
			      }
			      // if the device could NOT be opened
			      else{
				      clearStr(sendStatus, onStatus);
				      nullDev(sendStatus, onStatus); // calls the null device error message
				      break;
				      }

			      break;


                case GETMAXDATA:;


				// if device could be opened
				if(comedi_device != NULL){
					subdevice = atoi((par[3]));
					channel = atoi((par[4]));
					
					
					maxdata = comedi_get_maxdata(comedi_device, subdevice, channel);
					if(maxdata == 0){
						clearStr(sendStatus,onStatus);
						comediErr = comedi_strerror(comedi_errno());
						strcpy(sendStatus,comediErr);
						strcpy(onStatus ,onFailure);
						break;
					}
					else{

						clearStr(sendStatus,onStatus);
						snprintf(sendStatus,BUFFERSIZE,"%d",maxdata);
						strcpy(onStatus,onSuccess);
						break;
						}
				}
				// if the device could not be opened
				else{
					clearStr(sendStatus,onStatus);
					nullDev(sendStatus,onStatus);
					break;
				}
			  break;

		case GETRANGE:;
			      // if device could be opened
			      if(comedi_device != NULL){
				      subdevice = atoi((par[3]));
				      channel = atoi((par[4]));
					range = atoi((par[5]));

					rangeType = comedi_get_range(comedi_device, subdevice, channel, range);
					if(rangeType == NULL){
						clearStr(sendStatus,onStatus);
						comediErr = comedi_strerror(comedi_errno());
						strcpy (sendStatus,comediErr);
						strcpy(onStatus ,onFailure);
						break;
						}

					else{
						clearStr(sendStatus,onStatus);
						snprintf(sendStatus,BUFFERSIZE,"Min:%g, Max: %g",rangeType->min,rangeType->max);
						strcpy(onStatus,onSuccess);
											
						getRangeWasExecuted = 1;
						break;
						}
					}
			      // if the device could not be opened
			      else{
				      clearStr(sendStatus,onStatus);
				      nullDev(sendStatus,onStatus);
				      break;
				      }
			      break;
			      
                case GETVER:;

			    // if device could be opened
			    if(comedi_device != NULL){
				    int versionNo = 0;

				    versionNo = comedi_get_version_code(comedi_device);
				    if(versionNo < 0){
					    clearStr(sendStatus, onStatus);
					    comediErr = comedi_strerror(comedi_errno());
					    strcpy(sendStatus,comediErr);
					    strcpy(onStatus ,onFailure);
					    break;
					    }
				    else{
					    clearStr(sendStatus, onStatus);
					    snprintf(sendStatus,BUFFERSIZE,"version: 0x%08x",versionNo);
					    strcpy(onStatus,onSuccess);
					    break;
					    }
				    }
			    // if the device could not be opened
			    else{
				    clearStr(sendStatus,onStatus);
				    nullDev(sendStatus,onStatus);
				    break;
				    }
			    break;

                case GETDRIVER:;

			       // if device could be opened
				if(comedi_device != NULL){
				
				const char *driverName = (char *) malloc(BUFFERSIZE); 
				driverName = comedi_get_driver_name(comedi_device);
					if(driverName == NULL){
						clearStr(sendStatus,onStatus);
						comediErr= comedi_strerror(comedi_errno());
						strcpy(sendStatus,comediErr);
						strcpy(onStatus ,onFailure);
						break;
						}
					else{
						clearStr(sendStatus,onStatus);
						strcpy(sendStatus, driverName); 
						strcpy(onStatus,onSuccess);
						break;
						}
					}
			       // if the device could not be opened
				else{
					clearStr(sendStatus,onStatus);
					nullDev(sendStatus,onStatus);
					break;
				}
			  break;


		case TOPHYS:;


			    // if device could be opened
			    if((comedi_device != NULL) && (rangeType != NULL)){

				    data = atoi((par[2]));
				    maxdata = atoi((par[3]));

				    physUnit = comedi_to_phys(data,rangeType, maxdata);
				    
				    //COMEDI_OOR_NAN
				    if(isnan(physUnit)){
					    clearStr(sendStatus,onStatus);
					    comediErr = comedi_strerror(comedi_errno());
					    strcpy(sendStatus,comediErr);
					    strcpy(onStatus ,onFailure);
					    break;
					    }
				    else{
					    clearStr(sendStatus,onStatus);
					    snprintf(sendStatus,BUFFERSIZE,"%g",physUnit);
					    strcpy(onStatus,onSuccess);
					    break;
					    }
				    }
			    // if the device could not be opened
			    else{
				    clearStr(sendStatus,onStatus);
				    nullDev(sendStatus,onStatus);
				    break;
				}
			  break;

		case FROMPHYS:;

			      // if device could be opened
			      if((comedi_device != NULL) && (rangeType != NULL)){
				      physUnit = atof((par[2]));
				      maxdata = atoi((par[3]));
				      sampleUnit = comedi_from_phys(physUnit,rangeType, maxdata);
				      clearStr(sendStatus,onStatus);
				      snprintf(sendStatus,BUFFERSIZE,"%d",sampleUnit);
				      strcpy(onStatus,onSuccess);
				      break;
				      }
			      // if the device could not be opened
			      else{
				      clearStr(sendStatus,onStatus);
				      nullDev(sendStatus,onStatus);
				      break;
				      }
			      break;
			      


		case GETNSUBDEV:;

				// if the device could be opened 
				if(comedi_device != NULL){
					int nsubdev;
					nsubdev = comedi_get_n_subdevices(comedi_device);
					if (nsubdev < 0){
						clearStr(sendStatus,onStatus);
						comediErr = comedi_strerror(comedi_errno());
						strcpy(sendStatus,comediErr);
						strcpy(onStatus ,onFailure);
						break;
						}
					else{
						clearStr(sendStatus,onStatus);
						snprintf(sendStatus,BUFFERSIZE,"%d",nsubdev);
						strcpy(onStatus,onSuccess);
						break;
						}
					}
				// if the device could not be opened
				else{
					clearStr(sendStatus,onStatus);
					nullDev(sendStatus,onStatus);
					break;
				}
				break;
		case GETSUBDEVTYPE:;

				// if the device could be opened 
				if(comedi_device != NULL){
				      subdevice = atoi(par[3]);
				      
					int type;
					type = comedi_get_subdevice_type(comedi_device, subdevice);
					if (type < 0){
						clearStr(sendStatus,onStatus);
						comediErr = comedi_strerror(comedi_errno());
						strcpy(sendStatus,comediErr);
						strcpy(onStatus ,onFailure);
						break;
						}
					else{
						clearStr(sendStatus,onStatus);
						snprintf(sendStatus,BUFFERSIZE,"%d",type);
						strcpy(onStatus,onSuccess);
						break;
						}
					}
				// if the device could not be opened
				else{
					clearStr(sendStatus,onStatus);
					nullDev(sendStatus,onStatus);
					break;
				}
				break;

				
		case MODPROBE:;

			      int uidStatusSU, uidStatusU;
			      static int execStatusMod;
			      char *optionProbe = (char *)malloc(BUFFERSIZE);
			      char *driverProbe = (char *)malloc(BUFFERSIZE);
			      char *modelProbe = (char *)malloc(BUFFERSIZE);

			      clearStr(sendStatus,onStatus);


			      if(optionProbe != NULL)
				      strcpy(optionProbe, "driver");


			      if (strncmp(par[2], "-",1) != 0){ /* tests modprobe options */ 
				      //printf("server: Type the modprobe command option(-v, -r, -c, -n...refer to the command manual for more info)\n");
				      break;


				      }


			      if(driverProbe != NULL)
				      getDriverName(par[3],BUFFERSIZE, optionProbe, driverProbe); /* catches driver name from config file */

			       			       
			      if (driverProbe == NULL){ /* tests if driver was found from the model name */
				      //printf("server: No driver could be found for this device: %s\n",par[3]);
				      break;
			      }

			      
			      static char *argsMod[4] ;
			      argsMod[0] = "modprobe";
			      argsMod[1] = par[2];
			      argsMod[2] = driverProbe;
			      argsMod[3] = NULL;


			      /*
			       *
			       * Starts forking in 2 processes to call modprobe from the command line of the child process
			       * First fork() is called to create a child process. Inside the child process the setuid()
			       * command is called to give root power. This is needed for the modprobe command. 
			       * Right after that the close() commands are issued to close the stdout and sterr
			       * the dup command redirects the 2 sysouts to the socket. Error messages and out messages
			       * are now sent directly to the client. 
			       *
			       */


			      pid = fork();

			      if (pid == -1){
				      /*
				       * Error on fork():
				       */
				      fprintf(stderr,"%s: Failed to fork()\n", strerror(errno));
				      exit(13);
			      } else if (pid == 0){

				      /* 
				       * Child process                
				       */
				      //printf("PID %ld: Started child process.Parent PID is: %ld. \n",
					//	      (long) getpid(),   /* child process */
						//      (long) getppid()); /* parent process */

				      uid_t uIDsu = 0;        /* sets superuser ID */
				      uid_t uIDu = getuid();  /* gets user ID */ 
				      uidStatusSU = setuid(uIDsu); /* sets superuser ID to issue modprobe */
				     
				      if(uidStatusSU < 0){   /* checks setuid execution for superuser setting */
					     break;
					      
					      }
				      else{
					      close(1); /* closes stdout and redirects to socket */
					      close(2); /* closes the stdout and redirects to socket */ 
					      dup2(newSockfd,1);
					      dup2(newSockfd,2);

					      execStatusMod =  execCmd(MOD_PATH,argsMod);

					      if(execStatusMod != 0){

						      //printf("server: Modprobe execution error. Check sent messages\n"); 
						      break;
						      }


					      uidStatusU = setuid(uIDu); /* backs the user ID */
					      if(uidStatusU < 0){
						      close(newSockfd);
						      }
					      }
				     
					      }

			      else {
				      /* 
				       * Parent process 
				       */
				      //printf("PID %ld: child process starts in PID %ld. \n",
					//	      (long)getpid(), /* parent process */
						//      (long) pid);    /* child PID */

				      /*
				       * Waits for end of child process:
				       */
				      wpid = wait(&wstatus);  /* Status of child process exit */
				      if ( wpid == -1 ){
					      /* 
					       * If the wait() call fails: 
					       */
					      fprintf(stderr,"%s: wait() \n", strerror(errno));
					     // return 1;
				      }
				      else if (wpid != pid)
					      /* This situation should never occur */
					      abort();
				      else{
					      /*
					       * End of child process
					       */

					      if (WIFEXITED(wstatus)){ /* Using macros for the wait() test */
						     /*
						      * Normal exit 
						      */
						     //printf("Exit:  %d\n", WEXITSTATUS(wstatus));
						     if (WEXITSTATUS(wstatus) == 0){
							     clearStr(sendStatus,onStatus);
							     strcat(sendStatus,"Driver ");
							     strcat(sendStatus,driverProbe);
							     strcat(sendStatus, " configured.");
							     strcpy(onStatus,onSuccess);
							        
						     }
					     }
					      else if( WIFSIGNALED(wstatus) ){
						      

						     /*
						      * If the process aborts, was signaled or eliminated:
						      */
						     //printf("Signal : %d%s\n",WTERMSIG(wstatus), WCOREDUMP(wstatus) ? " with core file. " : "");
						     clearStr(sendStatus,onStatus);
						     strcat(sendStatus, "Driver ");
						     strcat(sendStatus, driverProbe);
						     strcat(sendStatus, "could not be loaded.");
						     strcpy(onStatus ,onFailure);
						    
						     }
					      else {
						     /* 
						      * Interrupted child process:
						      */
						     //printf("Stopped.\n");
					      }
				      }
			      }





			      free(optionProbe);
			      free(driverProbe);
			      free(modelProbe);

			      optionProbe = NULL;
			      driverProbe = NULL;
			      modelProbe = NULL;

			      break;


		case CONFIG:;

			    char *optionConf = (char *) malloc(BUFFERSIZE);
			    char *modelConf = (char *)malloc(BUFFERSIZE);
			    char *driverConf = (char *)malloc(BUFFERSIZE);

			    if(optionConf != NULL)
			    strcpy(optionConf, "model");

			    bzero(arg,sizeof arg);				
			    strcpy(arg,dev);
			    strcat(arg,par[3]);

			    clearStr(sendStatus,onStatus);


			    if (strncmp(par[2], "-",1) != 0){ /* tests comedi_config options */ 
				    //printf("server: Type comedi_config options (-v, -q, -i, -r...type 'comedi_config --help' for more info)\n");
				    break;

				    }

			    //printf("Arguments: %s,%s,%s\n",arg,par[2],par[4]);
			    if(modelConf != NULL)
				    getDriverName(par[4],BUFFERSIZE,optionConf, modelConf);

			    //printf("Model: %s\n", model);

			    static char *argsConf[5] ;
			    argsConf[0] = "comedi_config";
			    argsConf[1] = par[2];
			    argsConf[1] = arg;
			    argsConf[2] = modelConf;
			    argsConf[3] = par[5];
			    argsConf[4] = NULL;

			    pid = fork();
			    if (pid == -1){
				    
				    /*
				     * fork() error:
				     */

				    fprintf(stderr,"%s: Failed to fork()\n", strerror(errno));
				    exit(13);

				    } else if (pid == 0){
					    extern char **environ;
				      /* 
				       * Child process:
				       */
				      //printf("PID %ld: Child process started, parent process is: %ld. \n",
						//      (long) getpid(),   /* child process */
					//	      (long) getppid()); /* parent process */
			  
				    close(1); 
				    close(2);
				    dup2(newSockfd,1);
				    dup2(newSockfd,2);

				    //executes comedi_config. The argument are the device and model
				    //PNP boards don't need additional arguments. Non PNP boards need IO base address 
				    //and IRQ. Optional arguments are DMA channel, main clock speed, analog input
				    //range (min and max voltage, in microvolts), analog output range (min and max
				    //voltage, in microvolts), timer DMA mode.
				    //ex.: comedi_config /dev/comedi10 comedi_parport 0x370,5 (IO Base,IRQ)
				    //     comedi_config /dev/comedi1 das16 0x390,3,3,10 (IO Base, IRQ,DMA Mode, Clock).  
				    //Since the tokenizer does not separates tokens based on commas, the whole optionals string is
				    //treated as par[4].
				    

				    execStatusMod =  execCmd(CONF_PATH,argsConf);

				    if (execStatusMod != 0){

					    //printf("server: comedi_config error. Check messages sent to client\n");
					    }

				    }
			  

				      /* 
				       * Parent process:
				       */
				      //printf("PID %ld: Child process starts on PID %ld. \n",
				//		      (long)getpid(), /* parent process */
					//	      (long) pid);    /* child PID */

				      /*
				       * Waits for the end of the child process:
				       */
				      do {
					      wpid = wait(&wstatus);  /* Exit status of the child process */
					      //printf("Left child process w/ socket= %d & status: %d\n",newSockfd,WEXITSTATUS(wstatus));
				      } while (wpid != pid);

				      if ( wpid == -1 ){
					      /* 
					       * If the call to wait() fails: 
					       */

					      fprintf(stderr,"%s: wait() \n", strerror(errno));
					      
					      }

				      else if (wpid != pid)

					      /* This situation should never happen! */
					      abort();
				      else{
					      /*
					       * End of child process:
					       */
					      if (WIFEXITED(wstatus)){ /* Using macros for the wait() test */
						     /*
						      * Normal exit
						      */
						     //printf("Exit: %d\n", WEXITSTATUS(wstatus));
						     if (WEXITSTATUS(wstatus) == 0){
							     clearStr(sendStatus,onStatus);
							     strcat(sendStatus,"Device ");
							     strcat(sendStatus,modelConf);
							     strcat(sendStatus, " configured.");
							     //printf("Message if ok: '%s' \n",sendStatus);
							     strcpy(onStatus,onSuccess);
							  
							     }


					      } else if( WIFSIGNALED(wstatus) ){
						     /*
						      * If the child process aborted, was signaled or eliminated:
						      */
						     //printf("Socket %d\n",newSockfd);
						     //printf("Signal : %d\n", WTERMSIG(wstatus));
						     //printf("Right after signal\n");
						     //%s\n",WTERMSIG(wstatus), WCOREDUMP(wstatus) ? " with core file. " : "");
						     clearStr(sendStatus,onStatus);
						     strcat(sendStatus, "Device ");
						     strcat(sendStatus, modelConf);
						     strcat(sendStatus, " not configured.");

						     //printf("After signal and concatenations\n");
						     strcpy(onStatus ,onFailure);					     
						     
					      }else {
						     /* 
						      * If interrupted (child):
						      */
						     //printf("Stopped.\n");
					      }
				      }
			 


			    free(optionConf);
			    free(driverConf);
			    free(modelConf);

			    optionConf = NULL;
			    driverConf = NULL;
			    modelConf = NULL;

			    break;

			    }

	}
void nullDev(char *sendStr, char *rcStr){
	sendStr = "acqd: the comedi device is not opened. Try running the comedi_open function first.";
	rcStr = "1\n\n";

}

void clearStr(char *sendStr, char *rcStr){
	bzero(sendStr, BUFFERSIZE * sizeof(char));
	bzero(rcStr,BUFFERSIZE * sizeof(char));
}


