/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 * Comedi acquisition client: Gets driver version code for Comedi device.
 * 
 * Revision 1.1  2009/12/20  17:56:00 elaine
 * Initial revision
 *
 *
 * ------------------------------------------------------------------------------------------
 *
 */


/* required header files */
#include "../includes/stack-c.h" // provides Scilab stack functions
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants


/* Interface for building the following Scilab native:
 *    --> [version] = getBoardVersion (IP,DEV) 
 */

extern void getBoardVersion __PARAMS((char *ip, char *dev, char *recvVersion, int *m3));

int intGetBoardVersion(char *fname, unsigned long fname_len){

	static int pos1,pos2;
	static char *pos3;
	static int m1, m2, m3;
	static int n1, n2, n3;
	static char *varip = NULL;
	static char *vardev = NULL;
	

	/* checks number of arguments and return variables */
	static  int minlhs = 1, maxlhs = 1, minrhs = 2, maxrhs = 2;
	CheckRhs(minrhs, maxrhs); // Right Hand Side: min 2 args, max 2 args. 
	CheckLhs(minlhs, maxlhs); // Left Hand Side: min 1 args, max 1 args. 


	/* tests argument type */

	if ( GetType(1) != sci_strings ){
		Scierror(814, "%s: expecting a  string",fname);
		exit (0);
	}else{

	/* puts argument on the stack */
	/* receives RHS arguments. Fills dimensions in m1 and n1. pos1 is the pointer to the char stack (cstk) */
        GetRhsVar(1, "c", &m1, &n1, &pos1); 
	varip = strdup (cstk(pos1));
	
	}


	if ( GetType(2) != sci_strings) {
		Scierror(814, "%s: expecting a string",fname);
		exit(0);
	}else{
        GetRhsVar(2,"c", &m2, &n2, &pos2);
	vardev = strdup(cstk(pos2));
	}

	

	/* calls user function with values in the stack  */
	getBoardVersion(varip,vardev,&pos3,&m3);

	n3 = 1; // sets LHS parameter string size to one                                     

	// puts LHS strings on the stack
	CreateVarFromPtr(3,"c", &m3, &n3, &pos3);



         LhsVar(1) = 3;
	FreePtr(&pos3);
	

        return 0;

}






