/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 * Comedi acquisition client: Comedi device remote configuration function.
 * 
 * Revision 1.1  2009/06/29  00:00:00 elaine
 * Initial revision
 *
 * Revision 1.2  2009/07/21  00:00:00 elaine
 * Added blank strip functions and remote functions
 *
 * Revision 1.3  2009/11/04  00:00:00 elaine
 * Divided main code in separate functions and included im comedisrv.h header;
 * Solved C/Scilab string interfacing issue by using CreateVarFromPtr and removing PutLhs.
 *
 * Revision 1.4  2009/12/05  16:53:02 elaine
 * Minor enhancements
 *
 * ------------------------------------------------------------------------------------------
 *
 */


/* required header files */
#include "../includes/stack-c.h" // provides Scilab stack functions
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants


void confboard(char *ip, char *acq,char *dev, char *io, char *irq, char *opt,char **recvModprobe, int *m7, char **recvConfig, int *m8){


	int sentBytes, receiveStatus;
	int sockfd = -1, rc = -1;
	char *modprobeStat, *configStat, *modprobeMsg,*configMsg;
	
	
	printf("Opcao: %s\n",opt);

	if((modprobeStat = (char *) malloc ((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
		printf("intconf: Not enough space to allocate modprobeStat\n");
		exit(1);
		}

	if((configStat = (char *) malloc ((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("intconf: Not enough space to allocate configStat\n");
		exit(1);
		}

	if((modprobeMsg = (char *) malloc ((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("intconf: Not enough space to allocate modprobeMsg\n");
		exit(1);
	}

	if((configMsg = (char *) malloc((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("intconf: Not enough space to allocate configMsg\n");
		exit(1);
	}

	if((*recvModprobe = (char *) malloc((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("intconf: Not enough space to allocate recvModprobe\n");
		exit(1);
	}

	if((*recvConfig = (char *) malloc((unsigned) (BUFFERSIZE * sizeof(char)))) == NULL){
		printf("intconf: Not enough space to allocate recvConfig\n");
		exit(1);
	}


	/* tries to connect to server. On success sends acquisition request */
	if ((sockfd = connectToServer(ip)) > 0){
		remoteModprobe(acq,opt,modprobeMsg); // assembles message to send 
		if ((sentBytes = clientSender(sockfd,modprobeMsg)) >= 0){	// sends message to socket, returns number of bytes sent
			if ((receiveStatus = clientReceiver(sockfd,modprobeStat)) > 0) {  // receives message from server,returns number of bytes read and closes socket 
				rc = checkCmdStatus(modprobeStat,*recvModprobe); // strips message from HTML protocol, strips into return code and message
				if (rc == 0){
					
					*m7 = strlen(*recvModprobe);
					}
				if (rc == -1){
					bzero(*recvModprobe, BUFFERSIZE * sizeof(char));
					strcat(*recvModprobe, "It was not possible to load this driver!");
					strcat(*recvModprobe, acq);		 
					*m7 = strlen(*recvModprobe);
					return(0);
					}
			}
			else{
				bzero(*recvModprobe, BUFFERSIZE * sizeof(char));
				*recvModprobe = "It was not possible to receive message from server";
				*m7 = strlen(*recvModprobe);
				return(0);

				} // end of clientReceiver if
		}
		else{
			bzero(*recvModprobe, BUFFERSIZE * sizeof(char));
			strcat(*recvModprobe,"It was not possible to send message: ");
			strcat(*recvModprobe, modprobeMsg);
			*m7 = strlen(*recvModprobe);
			return(0);

			} // end of clientSender if
	}
	else{
		
		bzero(*recvModprobe, BUFFERSIZE * sizeof(char));
		strcat(*recvModprobe, "Open: It was not possible to open a connection with ");
		strcat(*recvModprobe, ip);		                
		*m7 = strlen(*recvModprobe);
		return(0);
	}//end of the connectToServer if 


	
// 	free(modprobeStat);
//	free(modprobeMsg);
//	free(recvModprobe);
//	modprobeStat = NULL;
//	modprobeMsg = NULL;
//	recvModprobe = NULL;
	


	if ((sockfd = connectToServer(ip)) >  0){
		remoteComediConfig(dev,acq,io,irq, opt,configMsg);  // assembles message to send
		if ((sentBytes = clientSender(sockfd, configMsg)) >= 0){ // sends message to socket, returns number of bytes sent
			
			if ((receiveStatus = clientReceiver(sockfd,configStat)) > 0) { // receives message from server. closes socket
				

				rc = checkCmdStatus(configStat,*recvConfig); //strips message from HTML protocol, strips into return code and message
				if (rc >= 0){
					*m8 = strlen(*recvConfig);
				}
				if (rc == -1){
					bzero(*recvConfig, BUFFERSIZE * sizeof(char));
					strcat(*recvConfig, "It was not possible to configure this board!");
					strcat(*recvConfig, acq);		 
					*m8 = strlen(*recvConfig);
					return(0);
					}
			}
			else{
				bzero(*recvConfig, BUFFERSIZE * sizeof(char));
				*recvConfig = "It was not possible to receive message from server";
				*m8 = strlen(*recvConfig);
				return(0);

			} // end of clientReceiver if
		}
		else{
			bzero(*recvConfig, BUFFERSIZE * sizeof(char));
			strcat(*recvConfig,"It was not possible to send message: ");
			strcat(*recvConfig, configMsg);
			*m8 = strlen(*recvConfig);
			return(0);
			} // end of clientSender if
	}
	else{
		bzero(*recvConfig, BUFFERSIZE * sizeof(char));
		strcat(*recvConfig, "Open: It was not possible to open a connection with ");
		strcat(*recvConfig, ip);		                
		*m8 = strlen(*recvConfig);
		return(0);
	}//end of the connectToServer if



//	printf("ANtes dos frees\n");
//	free(configStat);
//	free(configMsg);
//	free(recvConfig);
//	configStat = NULL;
//	configMsg = NULL;
//	recvConfig = NULL;
//	printf("Depois dos frees\n");



}

