/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * -----------------------------------------------------------------------------------------
 *  Comedi acquistion block: Computational function for the Comedi Analog Data Read block
 *
 *  Revision 1.1   2009/10/15  00:00:00 elaine
 *  Initial revision
 *
 *  Revision 1.2   2009/11/30  00:00:00 elaine
 *  Added comedi_data_read function and string conversion to integer.
 *
 *  Revision 1.3   2009/12/12  00:00:00 elaine
 *  Minor enhancements
 *
 *  Revision 1.4   2009/12/15  00:46:00 elaine
 *  Added other functions: comedi_get_maxdata, comedi_get_range and comedi_to_phys. 
 *
 *  Revision 1.5   2009/12/24  12:09:00 elaine
 *  Improved pointer allocation. Included error checking. 
 *
 *  Revision 1.6   2010/01/06  22:59:00 elaine
 *  Broke code in smaller functions (Comedi/Scilab functions).
 *
 *  ----------------------------------------------------------------------------------------
 */


/* required header files */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h> 
#include <fcntl.h> 
#include <ctype.h>
#include <sys/types.h> 
#include <asm/types.h> 
#include "../includes/scicos_block4.h"  /* Specific for scicos block developement */
#include "../includes/machine.h"
#include <math.h>
#include <comedilib.h>     /* Specific for Comedi Interface */
#include <sys/socket.h>    /* Specific for socket connection */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "../includes/comedisrv.h"


/* simulation flag constants */
#define Init           4   /* Initialization */ 
#define OutputUpdate   1
#define StateUpdate    2
#define Integration    0
#define Mode           9   /* Zero Crossing */
#define EventScheduler 3
#define Ending         5   




/* simulation function for the Acquisition of analog signals */
void comedi_net_an_in(scicos_block *block, int flag) {



static int convflag = 1;
int sockfd = -1 , sentBytes = 0, rc = -1, receiveStatus = 0;
int execStatus;
static char *openStat, *openMsg, *recvOpen;
static char *dataReadStat, *dataReadMsg, *recvDataRead;
static char *maxDataStat, *maxDataMsg, *recvMaxData;
static char *rangeStat, *rangeMsg, *recvRange;
static char *toPhysStat, *toPhysMsg, *recvToPhys;
static char *typeStat, *typeMsg, *recvType;
int subdevType;

	
	/* customer inputs from Scicos block */
	char ip[BUFFERSIZE];
	char dev[BUFFERSIZE];
	char subdev[BUFFERSIZE];
	char channel[BUFFERSIZE];
	char range[BUFFERSIZE];
	char aref[BUFFERSIZE];
	int typesAllowed[1] = {1};
	int typesCount = 1;

	/* Scicos input and output vectors */
	int *ipar = block->ipar;
	double *outptr = block->outptr[0];

	/* customer input sizes */
	int devSize, ipSize,subdevSize,channSize,rangeSize, arefSize;

	/* Scicos custom function to get strings from blocks to C programs */
	extern int cvstr_();



	
	

	switch(flag) {  

		case Init: // Initializes variables, starts memory allocation  


			
			/* initializes input parameter sizes coming from Scicos */ 
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			rangeSize = ipar[4];
			arefSize = ipar[5];
			
			
			/* initializes input parameters coming from Scicos */ 
			cvstr_(&ipSize,&ipar[6],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[6+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[6+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[6+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			cvstr_(&rangeSize, &ipar[6+devSize+ipSize+subdevSize+channSize], range, &convflag, rangeSize*sizeof(char));
			range[rangeSize] = '\0';
			
			cvstr_(&arefSize, &ipar[6+devSize+ipSize+subdevSize+channSize+rangeSize], aref, &convflag, arefSize*sizeof(char));
			aref[arefSize] = '\0';
			
			
			
                          /* memory allocation to character pointers */
                          
                          /* comedi_open */
			if ((openMsg  = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((openStat = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvOpen = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			
			
			
			/* comedi_get_maxdata */
			if ((maxDataStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((maxDataMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvMaxData = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			
			
			/* comedi_get_range */
			if ((rangeStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((rangeMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvRange = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			
			
			
						
		         /* comedi_data_read */
		         if((dataReadStat = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
			     set_block_error(-16);
                             			return;
                  		      }
                  		      if((dataReadMsg = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
			      set_block_error(-16);
                              			return;
                          }
                          if((recvDataRead = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
			     set_block_error(-16);
				return;
		      }
		      
		      
		      /* comedi_to_phys */
		      if((toPhysStat = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
			      set_block_error(-16);
				return;
		      }
		      if((toPhysMsg = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
			      set_block_error(-16);
				return;
		      }
		      if((recvToPhys = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
			      set_block_error(-16);
				return;
		      }
		      
		      
		      	
			/* comedi_get_subdevice_type */
			if ((typeStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((typeMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvType = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			
			openComediDevice(openStat, openMsg,recvOpen, ip, dev);

//			if ((sockfd = connectToServer(ip)) > 0){
//		
//
//				/* assembles message to send to server */
//				remoteComediOpen(dev,openMsg);
//
//				/* sends message to server. If the number of bytes is more than 0 receives 
//				 * message from server. If not, exits */
//				if((sentBytes = clientSender(sockfd,openMsg)) >=  0){
//					
//					/* receives msg from server. If the number of bytes read is more than 
//					 * 0 checks the return code from comedi_open executed remotely and the 
//					 * msg returned. If not, exits. */
//					if ((receiveStatus = clientReceiver(sockfd,openStat)) > 0){
//
//						/* checks the status of the comedi_open function. If status ok 
//						 * the program continues. If not ok program aborts */
//
//						//printf("client:valor do socket %d\n",sockfd);
//						rc = checkCmdStatus(openStat, recvOpen);
//						if (rc != 0){
//						Scierror(888,"Error on comedi_open.\n");
//				                  set_block_error(-1);
//							
//							}
//						//printf("comedi_open message: '%s'\n",recvOpen);
//
//						}
//					else{
//						Scierror(888,"It was not possible to receive message from server\n");
//				                  set_block_error(-1);
//						
//						}
//					}
//				else{
//				                  Scierror(888,"It was not possible to send message: '%s' to server\n",openMsg);
//				                  set_block_error(-1);
//				                  
//						}
//				close(sockfd);
//
//							
//
//			}
//			else{
//				
//				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
//				set_block_error(-1);
//				
//			}
			
		verifySubdeviceType(typeStat, typeMsg, recvType, ip, dev, subdev, typesAllowed, typesCount);
			// /*
//		       * checks subdevice
//	              	 */
//
//		      /* establishes a connection with the server. If not possible, exits */
//		      if ((sockfd = connectToServer(ip)) > 0){
//			      /* assembles message to send to server */
//			      remoteGetSubdeviceType(dev,subdev,typeMsg);
//			      /* sends message to server. If the number of bytes is more than 0
//			       * receiver message from server. If not, exits */
//			      if ((sentBytes = clientSender(sockfd, typeMsg)) >= 0){
//
//				      /* receives msg from server. If the number of bytes read is more
//				       * than 0 , checks the return code from comedi_get_subdevice_type executed 
//				       * remotely and the returned msg. If not, exits. */
//				      if ((receiveStatus = clientReceiver(sockfd,typeStat)) > 0){
//
//					      /* checks the status of the comedi_get_subdevice_type function. If ok
//					       * converts returned values. If not, the program aborts */
//					      				      
//					      rc = checkCmdStatus(typeStat, recvType);
//					      if (rc == 0){
//						      //printf("RECEBI: '%s'\n",recvType);
//						      subdevType = atoi (recvType);
//						      if (subdevType != COMEDI_SUBD_AI) {
//						          Scierror(888,"Incorrect subdevice type. Please check manual\n");
//						          	set_block_error(-1);
//						          	end_scicos_sim();
//						      
//						      }
//						          
//						    }
//					      else{
//						      Scierror(888,"Error in comedi_get_subdevice_type.\n");
//				               set_block_error(-1);
//						      }
//
//					      }
//				      else{
//					     Scierror(888,"It was not possible to receive msgs from the server\n");
//				               set_block_error(-1);
//				      }
//
//				      }
//
//			      else{
//				     Scierror(888,"It was not possible to send message: '%s' to server\n", typeMsg);
//				               set_block_error(-1);
//			      }
//
//			      close(sockfd);
//			      //printf("depois de fechar socket:%d\n",sockfd);
//
//		      }
//
//		      else{
//
//			      Scierror(888,"It was not possible to open a connection with %s.\n",ip);
//				set_block_error(-1);
//				end_scicos_sim();
//
//		      }
		      
			
			/*
			 * gets max data on channel 
			 */
			 
			     
getMaxData(maxDataStat, maxDataMsg,recvMaxData, ip, dev, subdev, channel);			 
//			/* establishes a connection with the server. If not possible, exits */
//			if ((sockfd = connectToServer(ip)) > 0){
//		
//
//				/* assembles message to send to server */
//				remoteComediGetMaxdata(dev,subdev, channel, maxDataMsg);
//
//				/* sends message to server. If the number of bytes is more than 0 receives 
//				 * message from server. If not, exits */
//				if((sentBytes = clientSender(sockfd,maxDataMsg)) >=  0){
//				
//				   //printf("CHANNEL: '%s'\n",channel);
//					
//					/* receives msg from server. If the number of bytes read is more than 
//					 * 0 checks the return code from comedi_get_maxdata executed remotely and the 
//					 * msg returned. If not, exits. */
//					if ((receiveStatus = clientReceiver(sockfd,maxDataStat)) > 0){
//
//						/* checks the status of the comedi_get_maxdata function. If status ok 
//						 * the program continues. If not ok program aborts */
//
//						//printf("client:valor do socket %d\n",sockfd);
//						rc = checkCmdStatus(maxDataStat, recvMaxData);
//				                  if (rc >= 0){
//				                  
//						    //  printf("RECEBI: '%s'\n",recvMaxData);
//						      
//						    }
//					      else{
//					      
//					      Scierror(888,"Error in comedi_get_maxdata.\n");
//				               set_block_error(-1);
//						     
//						      }
//
//					      }
//				      else{
//				      
//				      Scierror(888,"It was not possible to receive msgs from the server\n");
//				               set_block_error(-1);
//					    
//				      }
//
//				      }
//
//			      else{
//			      Scierror(888,"It was not possible to send message: '%s' to server\n", maxDataMsg);
//			      set_block_error(-1);
//				     
//			      }
//
//			      close(sockfd);
//
//							
//
//			}
//			else{
//			 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
//                			 set_block_error(-1);
//				
//			}
			
			
			
getRange(rangeStat, rangeMsg,recvRange, ip, dev, subdev, channel, range);			
			///*
//			 * gets channel range
//			 */
//			 
//			/* establishes a connection with the server. If not possible, exits */
//			if ((sockfd = connectToServer(ip)) > 0){
//		
//
//				/* assembles message to send to server */
//				remoteComediGetRange(dev,subdev, channel, range, rangeMsg);
//
//				/* sends message to server. If the number of bytes is more than 0 receives 
//				 * message from server. If not, exits */
//				if((sentBytes = clientSender(sockfd,rangeMsg)) >=  0){
//					
//					/* receives msg from server. If the number of bytes read is more than 
//					 * 0 checks the return code from comedi_get_range executed remotely and the 
//					 * msg returned. If not, exits. */
//					if ((receiveStatus = clientReceiver(sockfd,rangeStat)) > 0){
//
//						/* checks the status of the comedi_get_range function. If status ok 
//						 * the program continues. If not ok program aborts */
//
//						//printf("client:valor do socket %d\n",sockfd);
//						rc = checkCmdStatus(rangeStat, recvRange);
//				                  if (rc >= 0){
//				                  
//						      printf("GET RANGE: '%s'\n",recvRange);
//						      
//						    }
//					      else{
//						      Scierror(888,"Error in comedi_get_range.\n");
//				                        set_block_error(-1);
//						      }
//
//					      }
//				      else{
//					     Scierror(888,"It was not possible to receive msgs from the server\n");
//				               set_block_error(-1);
//					    
//				      }
//
//				      }
//
//			      else{
//				     Scierror(888,"It was not possible to send message: '%s' to server\n", maxDataMsg);
//				               set_block_error(-1);
//				     
//			      }
//
//			      close(sockfd);
//
//							
//
//			}
//			else{
//				 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
//				set_block_error(-1);
//				}
//			
			break;



  case OutputUpdate:

		      
		      /* assignment from Scicos block inputs to program variables */
		     	
				/* initializes input parameter sizes coming from Scicos */ 
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			rangeSize = ipar[4];
			arefSize = ipar[5];
			
			
			/* initializes input parameters coming from Scicos */ 
			cvstr_(&ipSize,&ipar[6],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[6+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[6+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[6+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			cvstr_(&rangeSize, &ipar[6+devSize+ipSize+subdevSize+channSize], range, &convflag, rangeSize*sizeof(char));
			range[rangeSize] = '\0';
			
			cvstr_(&arefSize, &ipar[6+devSize+ipSize+subdevSize+channSize+rangeSize], aref, &convflag, arefSize*sizeof(char));
			aref[arefSize] = '\0';
			
			
                          sockfd = -1;
                          
                         	      
    analogRead(dataReadStat, dataReadMsg,recvDataRead, ip, dev, subdev, channel, range,aref);                  
		      
                        /*
		       * reads data on specified channel
	              	 */

		   //   /* establishes a connection with the server. If not possible, exits */
//		      if ((sockfd = connectToServer(ip)) > 0){
//			      /* assembles message to send to server */
//			      if (dataReadMsg != NULL)
//			      remoteComediDataRead(dev,subdev,channel,range,aref,dataReadMsg);
//			      printf("mensagem enviada : %s\n",dataReadMsg);
//
//			      /* sends message to server. If the number of bytes is more than 0
//			       * receiver message from server. If not, exits */
//			      if ((sentBytes = clientSender(sockfd, dataReadMsg)) >= 0){
//
//				      /* receives msg from server. If the number of bytes read is more
//				       * than 0 , checks the return code from comedi_open executed 
//				       * remotely and the returned msg. If not, exits. */
//				      if ((receiveStatus = clientReceiver(sockfd,dataReadStat)) > 0){
//
//					      /* checks the status of the comedi_data_read function. If ok
//					       * converts returned values. If not, the program aborts */
//					     // printf("client: valor do socket fora::::::::::::::::::::::::::::::::::::::::: %d\n",sockfd);
//
//					      rc = checkCmdStatus(dataReadStat, recvDataRead);
//					      if (rc >= 0){
//						      //printf("RECEBI: '%s'\n",recvDataRead);
//						   
//						    }
//					      else{
//						      Scierror(888,"Error in comedi_data_read.\n");
//				               set_block_error(-1);
//						      }
//
//					      }
//				      else{
//				      Scierror(888,"It was not possible to receive msgs from the server\n");
//				               set_block_error(-1);
//					    
//				      }
//
//				      }
//
//			      else{
//				      	      Scierror(888,"It was not possible to send message: '%s' to server\n", dataReadMsg);
//				               set_block_error(-1);
//				      
//				    
//			      }
//
//			      close(sockfd);
//			     // printf("depois de fechar socket:%d\n",sockfd);
//
//		      }
//
//		      else{
//
//			     Scierror(888,"It was not possible to open a connection with %s.\n",ip);
//				set_block_error(-1);
//				end_scicos_sim();
//				
//		      }
//		      
		      
outptr[0] = comediToPhys(recvMaxData, recvDataRead, toPhysStat,toPhysMsg, recvToPhys, ip);

/*
		       * converts sample data into physical unit
	              	 */
	              	 
		      // /* establishes a connection with the server. If not possible, exits */
//		      if ((sockfd = connectToServer(ip)) > 0){
//			      /* assembles message to send to server */
//			      remoteComediToPhys(recvDataRead,recvMaxData, toPhysMsg);
//
//			      /* sends message to server. If the number of bytes is more than 0
//			       * receiver message from server. If not, exits */
//			      if ((sentBytes = clientSender(sockfd, toPhysMsg)) >= 0){
//
//				      /* receives msg from server. If the number of bytes read is more
//				       * than 0 , checks the return code from comedi_open executed 
//				       * remotely and the returned msg. If not, exits. */
//				      if ((receiveStatus = clientReceiver(sockfd,toPhysStat)) > 0){
//
//					      /* checks the status of the comedi_to_phys function. If ok
//					       * converts returned values. If not, the program aborts */
//					      //printf("client: valor do socket fora::::::::::::::::::::::::::::::::::::::::: %d\n",sockfd);
//
//					      rc = checkCmdStatus(toPhysStat, recvToPhys);
//					      if (rc >= 0){
//						     // printf("RECEBI: '%s'\n",recvToPhys);
//						      dataPhys = atof (recvToPhys);
//					     // printf("data: %g",dataPhys);
//						      outptr[0] = dataPhys;
//						    }
//					      else{
//						       Scierror(888,"Error in comedi_to_phys.\n");
//				               set_block_error(-1);
//						      }
//
//					      }
//				      else{
//
//					       Scierror(888,"It was not possible to receive msgs from the server\n");
//				               set_block_error(-1);
//					      
//					      
//				      }
//
//				      }
//
//			      else{
//				       Scierror(888,"It was not possible to send message: '%s' to server\n", toPhysMsg);
//				               set_block_error(-1);
//				      
//				  				  
//			      }
//
//			      close(sockfd);
//			   //   printf("depois de fechar socket:%d\n",sockfd);
//
//		      }
//
//		      else{
//
//			     Scierror(888,"It was not possible to open a connection with %s.\n",ip);
//				set_block_error(-1);
//				end_scicos_sim();
//				               
//			     		      }





		      break;

		      
  
  case Ending:

		      /* frees all allocated pointers */
		      
		      /* comedi_open */
		      free(openMsg);
		      openMsg = NULL;

		      free(openStat);
		      openStat = NULL;

		      free(recvOpen);
		      recvOpen = NULL;
		      

		      /* comedi_data_read */
		      free(dataReadStat);
		      dataReadStat = NULL;

		      free(dataReadMsg);
		      dataReadMsg = NULL;

		      free(recvDataRead);
		      recvDataRead = NULL;
		      
		      
		      /* comedi_get_maxdata */
		      free(maxDataStat);
		      maxDataStat = NULL;
		      
		      free(maxDataMsg);
		      maxDataMsg = NULL;
		      
		      free(recvMaxData);
		      recvMaxData = NULL;
		      
		      
		      /* comedi_get_range */
		      free(rangeStat);
		      rangeStat = NULL;
		      
		      free(rangeMsg);
		      rangeMsg = NULL;
		      
		      free(recvRange);
		      recvRange = NULL;
		      
		      /* comedi_to_phys */
		      free(toPhysStat);
		      toPhysStat = NULL;
		      
		      free(toPhysMsg);
		      toPhysMsg = NULL;
		      
		      free(recvToPhys);
		      recvToPhys = NULL;
		      
		            /* comedi_get_subdevice_type */
		      free(typeMsg);
		      typeMsg = NULL;

		      free(typeStat);
		      typeStat = NULL;

		      free(recvType);
		      recvType = NULL;
		      
		      
		      
      break;	  
        
  } // close the switch
   
} // close the function 

