/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * ------------------------------------------------------------------------------------------
 * Comedi acquisition client: Comedi device configuration interface for Scilab.
 * 
 * Revision 1.1  2009/06/29  00:00:00 elaine
 * Initial revision
 *
 * Revision 1.2  2009/07/21  00:00:00 elaine
 * Added blank strip functions and remote functions
 *
 * Revision 1.3  2009/11/04  00:00:00 elaine
 * Divided main code in separate functions and included im comedisrv.h header;
 * Solved C/Scilab string interfacing issue by using CreateVarFromPtr and removing PutLhs.
 *
 * Revision 1.4  2009/12/05  16:53:02 elaine
 * Minor enhancements
 *
 * ------------------------------------------------------------------------------------------
 *
 */


/* required header files */
#include "../includes/stack-c.h" // provides Scilab stack functions
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "../includes/comedisrv.h"  // provides comedi modified functions and constants


/* Interface for building the following Scilab native:
      --> [modprobeStat, configStat] = confboard (IP,ACQ,DEV,IO,IRQ,OPT) */

extern void confboard __PARAMS((char *ip, char *acq,char *dev, char *io, char *irq, char *opt, char *recvModprobe,int *m7,  char *recvConfig, int *m8));

int intconfboard(char *fname, unsigned long fname_len){

	static int pos1,pos2,pos3,pos4, pos5, pos6;
	static char *pos7, *pos8;
	static int m1, m2, m3, m4, m5, m6, m7, m8;
	static int n1, n2, n3, n4, n5, n6, n7, n8;
	static char *varip = NULL;
	static char *varacq = NULL;
	static char *vardev = NULL;
	static char *vario = NULL;
	static char *varirq = NULL;
	static char *varopt = NULL;
	static char *varmod = NULL;
	static char *varconf = NULL;


	/* checks number of arguments and return variables */
	static  int minlhs = 2, maxlhs = 2, minrhs = 3, maxrhs = 6;
	CheckRhs(minrhs, maxrhs); // Right Hand Side: min 3 args, max 6 args. 
	CheckLhs(minlhs, maxlhs); // Left Hand Side: min 2 args, max 2 args. 


	/* tests argument type */

	if ( GetType(1) != sci_strings ){
		Scierror(814, "%s: expecting a  string",fname);
		exit (0);
	}else{

	/* puts argument on the stack */
	/* receives RHS arguments. Fills dimensions in m1 and n1. pos1 is the pointer to the char stack (cstk) */
        GetRhsVar(1, "c", &m1, &n1, &pos1); 
	varip = strdup (cstk(pos1));
	
	}


	if ( GetType(2) != sci_strings) {
		Scierror(814, "%s: expecting a string",fname);
		exit(0);
	}else{
        GetRhsVar(2,"c", &m2, &n2, &pos2);
	varacq = strdup(cstk(pos2));
	}

	if ( GetType(3) != sci_strings) {
		Scierror(814, "%s: expecting a string",fname);
		exit(0);
	}else{
        GetRhsVar(3,"c", &m3, &n3, &pos3);
	vardev = strdup(cstk(pos3));
	}

	if ( GetType(4) != sci_strings) {
		Scierror(814, "%s: expecting a string",fname);
		exit(0);
	}else{
        GetRhsVar(4,"c", &m4, &n4, &pos4);
	vario = strdup(cstk(pos4));
	}

	if ( GetType(5) != sci_strings) {
		Scierror(814, "%s: expecting a  string",fname);
		exit(0);
	}else{
        GetRhsVar(5,"c", &m5, &n5, &pos5);
	varirq = strdup(cstk(pos5));
	}
	if ( GetType(6) != sci_strings) {
		Scierror(814, "%s: expecting a  string",fname);
		exit(0);
	}else{
        GetRhsVar(6,"c", &m6, &n6, &pos6);
	varopt = strdup(cstk(pos6));
	}



	/* calls user function with values in the stack  */
        confboard(varip,varacq,vardev,vario,varirq,varopt, &pos7,&m7,&pos8, &m8);

	n7 = n8 = 1; // sets LHS parameter string size to one                                     

	// puts LHS strings on the stack
	CreateVarFromPtr(7,"c", &m7, &n7, &pos7);
	CreateVarFromPtr(8, "c", &m8, &n8, &pos8);


        LhsVar(1) = 7;
	LhsVar(2) = 8;
	FreePtr(&pos7);
	FreePtr(&pos8);

        return 0;

}






